/**********************************************************************
	"Copyright 1990-2018 Brian MacWhinney. Use is subject to Gnu Public License
	as stated in the attached "gpl.txt" file."
*/


#define CHAT_MODE 1

#include "cu.h"
#include "c_curses.h"

#ifdef UNX
	#define RGBColor int
#endif

#if !defined(UNX)
#define _main flucalc_main
#define call flucalc_call
#define getflag flucalc_getflag
#define init flucalc_init
#define usage flucalc_usage
#endif

#define IS_WIN_MODE FALSE
#include "mul.h" 

extern char OverWriteFile;
extern char outputOnlyData;
extern char isLanguageExplicit;
extern struct tier *defheadtier;
extern struct tier *headtier;

struct flucalc_speakers {
	char isSpeakerFound;
	char isMORFound;
	char isPSDFound; // +/.

	char *fname;
	char *sp;
	char *ID;

	float tm;
	float morUtt;
	float morWords;

	float prolongation;	// :
	float broken_word;	// ^
	float block;		// ≠
	float PWR;			// ↫ ↫ pairs//Part Word Repetition; Sequences
	float PWRRU;		// ↫ - - ↫  // iterations
	float phono_frag;	// &+
	float WWR;			// how many times "the [/] the [/] the" occurs. In this case it is 1.
	float WWRRU;		// [/]		// Whole Word repetition iterations
	float phrase_repets;// <> [/]
	float word_revis;	// [//]
	float phrase_revis;	// <> [//]
	float pauses_cnt;	// (.)
	float pauses_dur;	// (2.4)
	float filled_pauses;// &-

	struct flucalc_speakers *next_sp;
} ;

static char flucalc_ftime;
static struct flucalc_speakers *sp_head;


void usage() {
	puts("FLUCALC creates a spreedsheet with a series of fluency measures.");
#ifdef UNX
	printf("FluCalc REQUIRES THE PRESENCE OF THE \"%%mor:\" TIER BY DEFAULT.\n");
	printf("TO USE ONLY THE MAIN SPEAKER TIER PLEASE USE \"-t%%mor:\" OPTION.\n");
#else
	printf("%c%cFluCalc REQUIRES THE PRESENCE OF THE \"%%mor:\" TIER BY DEFAULT.%c%c\n", ATTMARKER, error_start, ATTMARKER, error_end);
	printf("%c%cTO USE ONLY THE MAIN SPEAKER TIER PLEASE USE \"-t%%mor:\" OPTION.%c%c\n", ATTMARKER, error_start, ATTMARKER, error_end);
#endif
	printf("Usage: flucalc [%s] filename(s)\n",mainflgs());
	mainusage(FALSE);
#ifdef UNX
	printf("FluCalc REQUIRES THE PRESENCE OF THE \"%%mor:\" TIER BY DEFAULT.\n");
	printf("TO USE ONLY THE MAIN SPEAKER TIER PLEASE USE \"-t%%mor:\" OPTION.\n");
#else
	printf("%c%cFluCalc REQUIRES THE PRESENCE OF THE \"%%mor:\" TIER BY DEFAULT.%c%c\n", ATTMARKER, error_start, ATTMARKER, error_end);
	printf("%c%cTO USE ONLY THE MAIN SPEAKER TIER PLEASE USE \"-t%%mor:\" OPTION.%c%c\n", ATTMARKER, error_start, ATTMARKER, error_end);
#endif
	cutt_exit(0);
}

static struct flucalc_speakers *freespeakers(struct flucalc_speakers *p) {
	struct flucalc_speakers *ts;

	while (p) {
		ts = p;
		p = p->next_sp;
		if (ts->fname != NULL)
			free(ts->fname);
		if (ts->sp != NULL)
			free(ts->sp);
		if (ts->ID != NULL)
			free(ts->ID);
		free(ts);
	}
	return(NULL);
}

static void flucalc_error(char IsOutOfMem) {
	if (IsOutOfMem)
		fputs("ERROR: Out of memory.\n",stderr);
	sp_head = freespeakers(sp_head);
	cutt_exit(0);
}


static void flucalc_initTSVars(struct flucalc_speakers *ts) {
	ts->isMORFound = FALSE;
	ts->isPSDFound = FALSE;

	ts->tm		  = 0.0;
	ts->morUtt	  = 0.0;
	ts->morWords  = 0.0;

	ts->prolongation	= 0.0;
	ts->broken_word		= 0.0;
	ts->block			= 0.0;
	ts->PWR				= 0.0;
	ts->PWRRU			= 0.0;
	ts->phono_frag		= 0.0;
	ts->WWR				= 0.0;
	ts->WWRRU			= 0.0;
	ts->phrase_repets	= 0.0;
	ts->word_revis	    = 0.0;
	ts->phrase_revis    = 0.0;
	ts->pauses_cnt		= 0.0;
	ts->pauses_dur		= 0.0;
	ts->filled_pauses   = 0.0;
}

static struct flucalc_speakers *flucalc_FindSpeaker(char *fname, char *sp, char *ID, char isSpeakerFound) {
	struct flucalc_speakers *ts, *tsp;

	uS.remblanks(sp);
	for (ts=sp_head; ts != NULL; ts=ts->next_sp) {
		if (uS.mStricmp(ts->fname, fname) == 0) {
			if (uS.partcmp(ts->sp, sp, FALSE, FALSE)) {
				ts->isSpeakerFound = isSpeakerFound;
				return(ts);
			}
		}
	}
	if ((ts=NEW(struct flucalc_speakers)) == NULL)
		flucalc_error(TRUE);
	if ((ts->fname=(char *)malloc(strlen(fname)+1)) == NULL) {
		free(ts);
		flucalc_error(TRUE);
	}
	if (sp_head == NULL) {
		sp_head = ts;
	} else {
		for (tsp=sp_head; tsp->next_sp != NULL; tsp=tsp->next_sp) ;
		tsp->next_sp = ts;
	}
	ts->next_sp = NULL;
	strcpy(ts->fname, fname);
	if ((ts->sp=(char *)malloc(strlen(sp)+1)) == NULL) {
		flucalc_error(TRUE);
	}
	strcpy(ts->sp, sp);
	if (ID == NULL)
		ts->ID = NULL;
	else {
		if ((ts->ID=(char *)malloc(strlen(ID)+1)) == NULL)
			flucalc_error(TRUE);
		strcpy(ts->ID, ID);
	}
	ts->isSpeakerFound = isSpeakerFound;
	flucalc_initTSVars(ts);
	return(ts);
}

void init(char f) {
	int i;
	struct tier *nt;

	if (f) {
		sp_head = NULL;
		flucalc_ftime = TRUE;
		if (defheadtier != NULL) {
			if (defheadtier->nexttier != NULL)
				free(defheadtier->nexttier);
			free(defheadtier);
			defheadtier = NULL;
		}
		mor_initwords();
		outputOnlyData = TRUE;
		OverWriteFile = TRUE;
		combinput = TRUE;
	} else {
		if (flucalc_ftime) {
			flucalc_ftime = FALSE;
			if (chatmode) {
				maketierchoice("@ID:",'+',FALSE);
				maketierchoice("%mor:",'+',FALSE);
			} else {
				fprintf(stderr, "FluCalc can only run on CHAT data files\n\n");
				cutt_exit(0);
			}
			i = 0;
			for (nt=headtier; nt != NULL; nt=nt->nexttier) {
				if (nt->tcode[0] == '*') {
					i++;
				}
			}
			if (i != 1) {
				fprintf(stderr, "\nPlease specify only one speaker tier code with \"+t\" option.\n");
				cutt_exit(0);
			}
			for (i=0; GlobalPunctuation[i]; ) {
				if (GlobalPunctuation[i] == '!' ||
					GlobalPunctuation[i] == '?' ||
					GlobalPunctuation[i] == '.') 
					strcpy(GlobalPunctuation+i,GlobalPunctuation+i+1);
				else
					i++;
			}
			if (!f_override)
				stout = FALSE;
			AddCEXExtension = ".xls";
		}
	}
}

void getflag(char *f, char *f1, int *i) {
	f++;
	switch(*f++) {
		case 's':
			if (*f == '[' && *(f+1) == '-') {
				if (*(f-2) == '+')
					isLanguageExplicit = 2;
				maingetflag(f-2,f1,i);
			} else if ((*f == '[' && *(f+1) == '+') || ((*f == '+' || *f == '~') && *(f+1) == '[' && *(f+2) == '+')) {
				maingetflag(f-2,f1,i);
			} else {
				fprintf(stderr, "Please specify only postcodes, \"[+ ...]\", or precodes \"[- ...]\" with +/-s option.");
				cutt_exit(0);
			}
			break;
		case 't':
			if (*(f-2) == '+' && *f == '*') {
				maingetflag(f-2,f1,i);
			} else {
				fprintf(stderr, "\nPlease specify only one speaker tier code with \"+t\" option.\n");
				cutt_exit(0);
			}
			break;
		default:
			maingetflag(f-2,f1,i);
			break;
	}
}

static void flucalc_pr_result(void) {
	char  *sFName;
	float SLDr, TDr, X;
	struct flucalc_speakers *ts;

/*
	if (!stout) {
#ifdef _WIN32
		fprintf(fpout, "%s	Win95:CAfont:-15:0\n", FONTHEADER);
#else
		fprintf(fpout, "%s	CAfont:13:7\n", FONTHEADER);
#endif
	 } else {
		char  fontName[256];
		strcpy(fontName, "CAfont:13:7\n");
		cutt_SetNewFont(fontName, EOS);
	 }
*/
	if (sp_head == NULL) {
		fprintf(stderr,"\nERROR: No speaker matching +t option found\n\n");
	}
	fprintf(fpout, "%c%c%c", 0xef, 0xbb, 0xbf); // BOM UTF8
	fprintf(fpout,"File");
	fprintf(fpout,",Language,Corpus,Code,Age(Month),Sex,Group,SES,Role,Education,Custom_field");

	fprintf(fpout,",mor Utts");
	fprintf(fpout,",mor Words");
	fprintf(fpout,",Words/Min");

	fprintf(fpout,",# Prolongation");
	fprintf(fpout,",%% Prolongation");
	fprintf(fpout,",# Broken word");
	fprintf(fpout,",%% Broken word");
	fprintf(fpout,",# Block");
	fprintf(fpout,",%% Block");
	fprintf(fpout,",# PWR");
	fprintf(fpout,",%% PWR");
	fprintf(fpout,",# PWR RU");
//	fprintf(fpout,",%% PWR RU");
	fprintf(fpout,",# WWR");
	fprintf(fpout,",%% WWR");
	fprintf(fpout,",# WWR RU");
//	fprintf(fpout,",%% WWR RU");
	fprintf(fpout,",Mean RUs");
	fprintf(fpout,",# Phonological fragment");
	fprintf(fpout,",%% Phonological fragment");
	fprintf(fpout,",# Phrase repetitions");
	fprintf(fpout,",%% Phrase repetitions");
	fprintf(fpout,",# Word revisions");
	fprintf(fpout,",%% Word revisions");
	fprintf(fpout,",# Phrase revisions");
	fprintf(fpout,",%% Phrase revisions");
	fprintf(fpout,",# Pauses");
	fprintf(fpout,",%% Pauses");
//	fprintf(fpout,",# Pause duration");
//	fprintf(fpout,",%% Pause duration");
	fprintf(fpout,",# Filled pauses");
	fprintf(fpout,",%% Filled pauses");

	fprintf(fpout,",# SLD");
	fprintf(fpout,",%% SLD");
	fprintf(fpout,",# TD");
	fprintf(fpout,",%% TD");

	fprintf(fpout,",# Total (SLD+TD)");
	fprintf(fpout,",%% Total (SLD+TD)");

	fprintf(fpout,",SLD Ratio");

	fprintf(fpout,",Weighted SLD");

	fprintf(fpout, "\n");
	for (ts=sp_head; ts != NULL; ts=ts->next_sp) {
		if (!ts->isSpeakerFound) {
			fprintf(stderr,"\nWARNING: No data found for speaker \"%s\" in file \"%s\"\n\n", ts->sp, ts->fname);
			continue;
		}
// ".xls"
		sFName = strrchr(ts->fname, PATHDELIMCHR);
		if (sFName != NULL)
			sFName = sFName + 1;
		else
			sFName = ts->fname;
		outputStringForExcel(fpout, sFName, 0);
		if (ts->ID) {
			outputIDForExcel(fpout, ts->ID, -1);
		} else
			fprintf(fpout,",.,.,%s,.,.,.,.,.,.,.", ts->sp);
		if (!ts->isMORFound || ts->morWords == 0.0) {
			fprintf(stderr,"\n*** File \"%s\": Speaker \"%s\"\n", sFName, ts->sp);
			fprintf(stderr,"WARNING: Speaker \"%s\" has no \"%s\" tiers.\n\n", ts->sp, "%mor:");
			fprintf(fpout,",%.0f,%.0f,N/A,N/A,N/A,N/A,N/A,N/A,N/A,N/A,N/A,N/A,N/A,N/A,N/A,N/A,N/A,N/A,N/A,N/A,N/A,N/A,N/A,N/A,N/A,N/A,N/A,N/A,N/A,N/A,N/A,N/A,N/A,N/A,N/A\n", ts->morUtt, ts->morWords);
		} else {
			fprintf(fpout, ",%.0f", ts->morUtt);
			fprintf(fpout, ",%.0f", ts->morWords);
			if (ts->tm == 0.0)
				fprintf(fpout,",N/A");
			else
				fprintf(fpout,",%.3f", ts->morWords / (ts->tm / 60.0000));

			fprintf(fpout, ",%.3f", ts->prolongation);
			fprintf(fpout, ",%.3f", ts->prolongation/ts->morWords);
			fprintf(fpout, ",%.3f", ts->broken_word);
			fprintf(fpout, ",%.3f", ts->broken_word/ts->morWords);
			fprintf(fpout, ",%.3f", ts->block);
			fprintf(fpout, ",%.3f", ts->block/ts->morWords);
			fprintf(fpout, ",%.3f", ts->PWR);
			fprintf(fpout, ",%.3f", ts->PWR/ts->morWords);
			fprintf(fpout, ",%.3f", ts->PWRRU);
//			fprintf(fpout, ",%.3f", ts->PWRRU/ts->morWords);
			fprintf(fpout, ",%.3f", ts->WWR);
			fprintf(fpout, ",%.3f", ts->WWR/ts->morWords);
			fprintf(fpout, ",%.3f", ts->WWRRU);
//			fprintf(fpout, ",%.3f", ts->WWRRU/ts->morWords);
			X = ts->PWR + ts->WWR;
			if (X <= 0.0)
				X = 0.0;
			else {
				X = (ts->PWRRU + ts->WWRRU) / X;
			}
			fprintf(fpout, ",%.3f", X);
			fprintf(fpout, ",%.3f", ts->phono_frag);
			fprintf(fpout, ",%.3f", ts->phono_frag/ts->morWords);
			fprintf(fpout, ",%.3f", ts->phrase_repets);
			fprintf(fpout, ",%.3f", ts->phrase_repets/ts->morWords);
			fprintf(fpout, ",%.3f", ts->word_revis);
			fprintf(fpout, ",%.3f", ts->word_revis/ts->morWords);
			fprintf(fpout, ",%.3f", ts->phrase_revis);
			fprintf(fpout, ",%.3f", ts->phrase_revis/ts->morWords);
			fprintf(fpout, ",%.3f", ts->pauses_cnt);
			fprintf(fpout, ",%.3f", ts->pauses_cnt/ts->morWords);
//			fprintf(fpout, ",%.3f", ts->pauses_dur);
//			fprintf(fpout, ",%.3f", ts->pauses_dur/ts->morWords);
			fprintf(fpout, ",%.3f", ts->filled_pauses);
			fprintf(fpout, ",%.3f", ts->filled_pauses/ts->morWords);

			SLDr = ts->prolongation+ts->broken_word+ts->block+ts->PWR+ts->phono_frag+ts->WWR;
			fprintf(fpout, ",%.3f", SLDr);
			fprintf(fpout, ",%.3f", SLDr/ts->morWords);

			TDr = ts->phrase_repets+ts->word_revis+ts->phrase_revis+ts->pauses_cnt+ts->pauses_dur+ts->filled_pauses;
			fprintf(fpout, ",%.3f", TDr);
			fprintf(fpout, ",%.3f", TDr/ts->morWords);

			fprintf(fpout, ",%.3f", SLDr+TDr);
			fprintf(fpout, ",%.3f", (SLDr/ts->morWords)+(TDr/ts->morWords));

			if ((SLDr+TDr) == 0.0)
				fprintf(fpout, ",N/A");
			else
				fprintf(fpout, ",%.3f", SLDr / (SLDr+TDr));

//			SLDr = (((ts->PWRRU + ts->PWR) * (ts->PWRRU / ts->PWR)) + (2.0000 * (ts->prolongation + ts->block))) / ts->morWords;
//			SLDr = (((ts->iterations + ts->sequences) * (ts->iterations / ts->sequences)) + (2.0000 * (ts->prolongation + ts->block))) / ts->morWords;
//			SLDr = ( ( ts->PWR + ts->WWR ) * ( ( ts->PWRRU + ts->WWRRU ) / ( ts->PWR + ts->WWR ) ) ) + (2 * ( ts->prolongation + ts->block ) );

			X = (ts->PWR * 100.0000 / ts->morWords) + (ts->WWR * 100.0000 / ts->morWords);
			if (X > 0) {
				SLDr = (X * ( ( (ts->PWRRU * 100.0000 / ts->morWords) + (ts->WWRRU * 100.0000 / ts->morWords) ) / X) ) +
				       (2 * ( (ts->prolongation * 100.0000 / ts->morWords) + (ts->block * 100.0000 / ts->morWords) ) );
			} else {
//				SLDr = (2 * ( (ts->prolongation * 100.0000 / ts->morWords) + (ts->block * 100.0000 / ts->morWords) ) ) / ts->morWords;
				SLDr = (2 * ( (ts->prolongation * 100.0000 / ts->morWords) + (ts->block * 100.0000 / ts->morWords) ) );
			}
/*
If X is not equal to zero, then

1. ( X * ( ( PWRRU + WWRRU) / X  ) ) + ( 2 * ( prolongation + block ) )

If X is equal to zero, then

2. ( 2 * ( prolongation + block ) ) / MOR words

In this formula X = ( PWR + WWR )
*/


			fprintf(fpout, ",%.3f", SLDr);

			fprintf(fpout, "\n");
		}
	}
	sp_head = freespeakers(sp_head);
}

static char isUttDel(char *line, int pos) {
	if (line[pos] == '?' && line[pos+1] == '|')
		;
	else if (uS.IsUtteranceDel(line, pos)) {
		if (!uS.atUFound(line, pos, &dFnt, MBF))
			return(TRUE);
	}
	return(FALSE);
}

static char isOnlyOneWordPreCode(char *line, int wi) {
	int  i, wCnt;
	char word[BUFSIZ+1];

	findWholeScope(wi, templineC);
	uS.remFrontAndBackBlanks(templineC);
	if (templineC[0] == EOS)
		return(TRUE);
	wCnt = 0;
	i = 0;
	while ((i=getword(utterance->speaker, templineC, word, NULL, i))) {
		if (word[0] != '[')
			wCnt++;
	}
	if (wCnt <= 1)
		return(TRUE);
	else
		return(FALSE);
}

static char isPreviousItemWWR(char *line, int wi) {

	for (wi--; wi >= 0 && (isSpace(line[wi]) || line[wi] == '\n'); wi--) ;
	if (wi < 3)
		return(FALSE);
	if ((isSpace(line[wi-3]) || line[wi-3] == '\n') && line[wi-2] == '[' && line[wi-1] == '/' && line[wi] == ']')
		return(TRUE);
	return(FALSE);
}

static float roundFloat(double num) {
	long t;

	t = (long)num;
	num = num - t;
	if (num > 0.5)
		t++;
	return(t);
}

static int countSyllables(char *word) {
	int i, j, startIndex, vCnt;
	char tWord[BUFSIZ+1], isRepSeg, isBullet;

	if (word[0] == '[' || word[0] == '(')
		return(0);
	isRepSeg = FALSE;
	isBullet = FALSE;
	j = 0;
	for (i=0; word[i] != EOS; i++) {
		if (UTF8_IS_LEAD((unsigned char)word[i]) && word[i] == (char)0xE2) {
			if (word[i+1] == (char)0x86 && word[i+2] == (char)0xAB) {
				isRepSeg = !isRepSeg;
			}
		} else if (word[i] == HIDEN_C)
			isBullet = !isBullet;
		if (!isRepSeg && !isBullet && isalnum(word[i]))
			tWord[j++] = word[i];
	}
	tWord[j] = EOS;
	if (tWord[0] == EOS)
		return(0);
//	if (uS.mStricmp(tWord,"something") == 0)
//		return(2);
	vCnt = 0;
	if (uS.mStrnicmp(tWord,"some", 4) == 0) {
		vCnt = 1;
		startIndex = 4;
	} else {
		vCnt = 0;
		startIndex = 0;
	}
	for (i=startIndex; tWord[i] != EOS; i++) {
		if ((i == startIndex || !uS.isVowel(tWord+i-1)) && uS.isVowel(tWord+i)) {
			if ((tWord[i] == 'e' || tWord[i] == 'E') && tWord[i+1] == EOS) {
				if (vCnt == 0)
					vCnt++;
			} else
				vCnt++;
		}
	}
	return(vCnt);
}

void call()	{		/* tabulate array of word lengths */
	int  i, j, wi, vCnt;
	char word[BUFSIZ+1];
	char isPSDFound, curPSDFound, isRepSeg, isWWRFound;
	long stime, etime;
	double tNum;
	struct flucalc_speakers *ts;
	extern int getc_cr_lc;
	extern char fgets_cr_lc, Tspchanged, contSpeaker[];

	ts = NULL;
	currentatt = 0;
	currentchar = (char)getc_cr(fpin, &currentatt);
	while (getwholeutter()) {
/*
 if (lineno > tlineno) {
	tlineno = lineno + 200;
 }
 */
		if (*utterance->speaker == '@') {
			if (uS.partcmp(utterance->speaker,"@ID:",FALSE,FALSE)) {
				if (isIDSpeakerSpecified(utterance->line, templineC, TRUE)) {
					uS.remblanks(utterance->line);
					flucalc_FindSpeaker(oldfname, templineC, utterance->line, FALSE);
				}
			}
		} else if (*utterance->speaker == '*') {
			strcpy(templineC, utterance->speaker);
			ts = flucalc_FindSpeaker(oldfname, templineC, NULL, TRUE);
			if (ts != NULL) {
				for (i=0; utterance->line[i] != EOS; i++) {

					if (utterance->line[i] == HIDEN_C && isdigit(utterance->line[i+1])) {
						if (getMediaTagInfo(utterance->line+i, &stime, &etime)) {
							tNum = etime;
							tNum = tNum - stime;
							tNum = tNum / 1000.0000;
							ts->tm = ts->tm + roundFloat(tNum);
						}
					}

					if ((i == 0 || uS.isskip(utterance->line,i-1,&dFnt,MBF)) && utterance->line[i] == '+' &&
						uS.isRightChar(utterance->line,i+1,',',&dFnt, MBF) && ts->isPSDFound) {
						if (ts->morUtt > 0.0)
							ts->morUtt--;
						ts->isPSDFound = FALSE;
					}
				}
				isWWRFound = 0;
				i = 0;
				while ((i=getword(utterance->speaker, utterance->line, word, &wi, i))) {
					isRepSeg = FALSE;

					vCnt = countSyllables(word);

					for (j=0; word[j] != EOS; j++) {
						if (word[j] == ':')
							ts->prolongation++;
						else if (word[j] == '^')
							ts->broken_word++;
						else if (UTF8_IS_LEAD((unsigned char)word[j]) && word[j] == (char)0xE2) {
							if (word[j+1] == (char)0x89 && word[j+2] == (char)0xA0) // ≠
								ts->block++;
							else if (word[j+1] == (char)0x86 && word[j+2] == (char)0xAB) { // ↫ - - ↫
								isRepSeg = !isRepSeg;
								if (isRepSeg) {
									ts->PWRRU++;
									ts->PWR++;
								}
							}
						} else if (word[j] == '&' && word[j+1] == '+') {
							ts->phono_frag++;
						} else if (word[j] == '&' && word[j+1] == '-') {
							ts->filled_pauses++;
						}
						if (isRepSeg && word[j] == '-') // ↫ - - ↫
							ts->PWRRU++;
					}
					if (!strcmp(word, "[/]")) {
						if (isOnlyOneWordPreCode(utterance->line, wi)) {
							if (isWWRFound == 0)
								isWWRFound = 1;
							ts->WWRRU++;
						} else
							ts->phrase_repets++;
					} else {
						if (!isPreviousItemWWR(utterance->line, wi))
							isWWRFound = 0;

						if (!strcmp(word, "[//]")) {
							if (isOnlyOneWordPreCode(utterance->line, wi))
								ts->word_revis++;
							else
								ts->phrase_revis++;
						} else if (word[0] == '(' && uS.isPause(word, 0, NULL, &j)) {
							ts->pauses_cnt++;
							ts->pauses_dur = ts->pauses_dur + getPauseTimeDuration(word);
						}
					}
					if (isWWRFound == 1) {
						ts->WWR++;
						isWWRFound = 2;
					}
				}
			}
			strcpy(spareTier1, utterance->line);
		} else if (*utterance->speaker == '%' && ts != NULL) {
			ts->isMORFound = TRUE;
			isPSDFound	= ts->isPSDFound;
			curPSDFound = FALSE;
			i = 0;
			while ((i=getword(utterance->speaker, uttline, word, NULL, i))) {
				if (strchr(word, '|') != NULL) {
					ts->morWords++;
				} else {
					for (j=0; word[j] != EOS; j++) {
						if (isUttDel(word, j)) {
							ts->morUtt = ts->morUtt + (float)1.0;
							break;
						}
					}
				}
			}
		}
	}
}

CLAN_MAIN_RETURN main(int argc, char *argv[]) {
	isWinMode = IS_WIN_MODE;
	CLAN_PROG_NUM = FLUCALC;
	chatmode = CHAT_MODE;
	OnlydataLimit = 1;
	UttlineEqUtterance = FALSE;
	bmain(argc,argv,flucalc_pr_result);
	sp_head = freespeakers(sp_head);
}
